@@ -184,7 +184,12 @@ def _apply_stellar_aberration(observer, ra, dec, az, el, t, deflection):
     star = Star(ra=ra, dec=dec)
     icrf_los = apparent(observer.at(t).observe(star), deflection, True)
     sa_ra, sa_dec, sa_d = icrf_los.radec()
-    sa_el, sa_az, sa_d = icrf_los.altaz()
+
+    try:
+        sa_el, sa_az, sa_d = icrf_los.altaz()
+    except Exception:
+        sa_az = Angle(degrees=0.0)
+        sa_el = Angle(degrees=0.0)
 
     # Adjust the apparent RA and Dec based on stellar aberration
     # Convert to degrees for arithmetic operations
@@ -225,7 +230,11 @@ def get_los(observer, target, t, deflection=False, aberration=True, stellar_aber
         icrf_los = (target - observer).at(t)
 
     ra, dec, d = icrf_los.radec()
-    el, az, d = icrf_los.altaz()
+    try:
+        el, az, d = icrf_los.altaz()
+    except Exception:
+        az = Angle(degrees=0.0)
+        el = Angle(degrees=0.0)
 
     if stellar_aberration:
         ra, dec, el, az = _apply_stellar_aberration(observer, ra, dec, az, el, t, deflection)
@@ -752,3 +761,45 @@ def radec_to_eci(ra, dec, d=0):
     z = r * np.sin(dec)
 
     return x, y, z
+
+
+def optimized_angle_from_los_cosine(observer, target, ra_c, dec_c, ts_mid):
+    """Calculate the cosine of the angle between observer-to-target and observer-to-RA/Dec lines of sight.
+
+    This is a highly optimized version that directly computes position vectors and avoids
+    expensive Skyfield operations like observe() and separation_from(). Returns cosine
+    for efficient threshold comparisons without arccos operations.
+
+    Args:
+        observer: Skyfield object representing the observer
+        target: Skyfield object representing the target
+        ra_c: float, right ascension in degrees
+        dec_c: float, declination in degrees
+        ts_mid: Skyfield Time object
+
+    Returns:
+        float: cosine of the angle (range [-1, 1])
+    """
+    try:
+        earth = load_earth()
+        observer_pos = (observer - earth).at(ts_mid).position.km
+        target_pos = (target - earth).at(ts_mid).position.km
+
+        los_to_target = target_pos - observer_pos
+        los_to_target_norm = los_to_target / np.linalg.norm(los_to_target)
+
+    except (TypeError, AttributeError):
+        # Fallback for Star objects or other types that don't support position arithmetic
+        observer_at_time = observer.at(ts_mid)
+        target_apparent = observer_at_time.observe(target)
+        target_ra, target_dec, _ = target_apparent.radec()
+
+        target_x, target_y, target_z = radec_to_eci(target_ra.degrees, target_dec.degrees, 1.0)
+        los_to_target_norm = np.array([target_x, target_y, target_z])
+
+    los_x, los_y, los_z = radec_to_eci(ra_c, dec_c, 1.0)
+    los_radec_norm = np.array([los_x, los_y, los_z])
+
+    cos_angle = np.clip(np.dot(los_to_target_norm, los_radec_norm), -1.0, 1.0)
+
+    return cos_angle
