@@ -4,7 +4,7 @@ import tensorflow as tf
 import numpy as np
 
 from satsim.geometry.transform import rotate_and_translate
-from satsim.util import get_semantic_version
+from satsim.util import is_tensorflow_running_on_cpu
 from satsim.math import fftconv2p
 
 
@@ -253,19 +253,18 @@ def add_counts(fpa, r, c, cnt, r_offset=0, c_offset=0):
     Returns:
         A `Tensor`, a reference to the modified input `fpa` image
     """
-    r = tf.cast(r, tf.int32)
-    c = tf.cast(c, tf.int32)
+    r = tf.cast(r, tf.int32) + tf.cast(r_offset, tf.int32)
+    c = tf.cast(c, tf.int32) + tf.cast(c_offset, tf.int32)
 
-    # TODO no bounds checking in TF-GPU if fpa is int32 type
-    # h,w = fpa.get_shape().as_list()
-    # valid = (r >= 0) & (r < h) & (c >= 0) & (c < w)
-    # r = tf.boolean_mask(r, valid)
-    # c = tf.boolean_mask(c, valid)
-    # cnt = tf.boolean_mask(tf.convert_to_tensor(cnt, dtype=fpa.dtype), valid)
+    # fix for no bounds checking if fpa is int32 or if running CPU
+    if is_tensorflow_running_on_cpu() or fpa.dtype == 'int32':
+        h, w = fpa.get_shape().as_list()
+        valid = (r >= 0) & (r < h) & (c >= 0) & (c < w)
+        r = tf.boolean_mask(r, valid)
+        c = tf.boolean_mask(c, valid)
+        cnt = tf.boolean_mask(tf.convert_to_tensor(cnt, dtype=fpa.dtype), valid)
 
-    # r_offset = r_offset
-    # c_offset = c_offset
-    rc = tf.stack([r + r_offset, c + c_offset], axis=1)
+    rc = tf.stack([r, c], axis=1)
     return tf.compat.v1.tensor_scatter_nd_add(fpa, rc, cnt)
 
 
@@ -396,7 +395,7 @@ def transform_and_add_counts(fpa, r, c, cnt, t_start, t_end, t_osf, rotation, tr
     def func_eval(i, img, r_batch, c_batch, cnt_os_batch):
 
         # vectorize
-        tt = repeat(tf.linspace(t_start, t_end, t_osf), batch_size)
+        tt = tf.repeat(tf.linspace(t_start, t_end, t_osf), batch_size)
         rr = tf.tile(r_batch[i, :], [t_osf])
         cc = tf.tile(c_batch[i, :], [t_osf])
         cnt_os = tf.tile(cnt_os_batch[i, :], [t_osf])
@@ -425,26 +424,6 @@ def _to_batch_1d(x, batch_size):
     return x
 
 
-def _repeat_legacy_tf(input, repeats, axis=None, name=None):
-
-    from tensorflow.python.ops.ragged.ragged_util import repeat
-
-    if axis is None:
-        input = tf.reshape(input, [-1])
-        axis = 0
-    return repeat(input, repeats, axis, name)
-
-
-#
-# use tf.repeat if implemented
-#
-tf_ver = get_semantic_version(tf)
-if tf_ver[0] == 1 and tf_ver[1] < 15 or tf_ver[0] == 2 and tf_ver[1] == 0:
-    repeat = _repeat_legacy_tf
-else:
-    repeat = tf.repeat
-
-
 def _to_shape(a, b):
     y_, x_ = b.shape
     y, x = a.shape
