@@ -35,6 +35,8 @@ from satsim.geometry.astrometric import (
     get_los,
     get_los_azel,
     get_analytical_los,
+    load_earth,
+    optimized_angle_from_los_cosine,
 )
 from satsim.geometry.greatcircle import GreatCircle
 from skyfield.toposlib import _ltude
@@ -478,6 +480,12 @@ def image_generator(ssp, output_dir='.', output_debug=False, dir_debug='./Debug'
     if 'enable_stellar_aberration' not in ssp['sim']:
         ssp['sim']['enable_stellar_aberration'] = True
 
+    if 'enable_fov_filter' not in ssp['sim']:
+        ssp['sim']['enable_fov_filter'] = False
+
+    if 'fov_filter_radius' not in ssp['sim']:
+        ssp['sim']['fov_filter_radius'] = None
+
     if star_mode == 'bins':
         star_dn = ssp['geometry']['stars']['mv']['density']
         star_pe = mv_to_pe(zeropoint, ssp['geometry']['stars']['mv']['bins']) * t_exposure
@@ -555,18 +563,28 @@ def image_generator(ssp, output_dir='.', output_debug=False, dir_debug='./Debug'
         ts_collect_end = time.utc_from_list(tt, t_frame * num_frames)
 
         # site
+        site_mode = None
         if 'site' in ssp['geometry']:
 
             # note: stars will track horizontally where zenith is pointed up. focal plane rotation is simulated with the `rotation` variable
             star_rot = ssp['geometry']['site']['gimbal']['rotation']
             track_mode = ssp['geometry']['site']['track']['mode']
-            lat = ssp['geometry']['site']['lat']
-            lon = ssp['geometry']['site']['lon']
-            astrometrics['lat'] = float(_ltude(lat, 'latitude', 'N', 'S')) if isinstance(lat, str) else float(lat)
-            astrometrics['lon'] = float(_ltude(lon, 'longitude', 'E', 'W')) if isinstance(lon, str) else float(lon)
-            astrometrics['alt'] = float(ssp['geometry']['site'].get('alt', 0))
             astrometrics['track_mode'] = _parse_track_mode(track_mode, 0, num_frames)
-            observer = create_topocentric(astrometrics['lat'], astrometrics['lon'], astrometrics['alt'])
+
+            if 'tle' in ssp['geometry']['site']:
+                observer = create_sgp4(ssp['geometry']['site']['tle'][0], ssp['geometry']['site']['tle'][1])
+                site_mode = 'space'
+            elif 'tle1' in ssp['geometry']['site']:
+                observer = create_sgp4(ssp['geometry']['site']['tle1'], ssp['geometry']['site']['tle2'])
+                site_mode = 'space'
+            else:
+                lat = ssp['geometry']['site']['lat']
+                lon = ssp['geometry']['site']['lon']
+                astrometrics['lat'] = float(_ltude(lat, 'latitude', 'N', 'S')) if isinstance(lat, str) else float(lat)
+                astrometrics['lon'] = float(_ltude(lon, 'longitude', 'E', 'W')) if isinstance(lon, str) else float(lon)
+                astrometrics['alt'] = float(ssp['geometry']['site'].get('alt', 0))
+                observer = create_topocentric(astrometrics['lat'], astrometrics['lon'], astrometrics['alt'])
+                site_mode = 'ground'
 
             if 'tle' in ssp['geometry']['site']['track']:
                 track = create_sgp4(ssp['geometry']['site']['track']['tle'][0], ssp['geometry']['site']['track']['tle'][1])
@@ -673,6 +691,19 @@ def image_generator(ssp, output_dir='.', output_debug=False, dir_debug='./Debug'
             t_end_star = t_end
             t_frame_track_start = _parse_start_track_time(track_mode, frame_num, num_frames, ts_collect_start, ts_start)
 
+            # sensor position and velocity
+            if site_mode == 'space':
+                ts_mid = time.mid(ts_start, ts_end)
+                eci_sv = (observer - load_earth()).at(ts_mid)
+                pos = eci_sv.position.km
+                vel = eci_sv.velocity.km_per_s
+                astrometrics['x'] = float(pos[0])
+                astrometrics['y'] = float(pos[1])
+                astrometrics['z'] = float(pos[2])
+                astrometrics['vx'] = float(vel[0])
+                astrometrics['vy'] = float(vel[1])
+                astrometrics['vz'] = float(vel[2])
+
             # calculate object pixels
             r_obs_os, c_obs_os, pe_obs_os, obs_os_pix, obs_model = _gen_objects(ssp, render_mode,
                                                                                 obs, obs_cache,
@@ -1036,6 +1067,55 @@ def _gen_objects(ssp, render_mode,
     ts_mid = time.utc_from_list(tt, t_start + 0.5 * (t_end - t_start))
     ts_end = time.utc_from_list(tt, t_end)
 
+    enable_deflection = ssp['sim']['enable_deflection']
+    enable_light_transit = ssp['sim']['enable_light_transit']
+    enable_stellar_aberration = ssp['sim']['enable_stellar_aberration']
+    enable_fov_filter = ssp['sim'].get('enable_fov_filter', False)
+    fov_filter_radius = ssp['sim'].get('fov_filter_radius')
+
+    az_arr = None
+    el_arr = None
+    ra_c = None
+    dec_c = None
+    fov_half_diag_pad_cos = None
+
+    if enable_fov_filter:
+        if track_mode == 'fixed':
+            ra_c, dec_c, _, _, _, _ = get_los_azel(
+                observer,
+                az_arr[1],
+                el_arr[1],
+                ts_mid,
+                deflection=enable_deflection,
+                aberration=enable_light_transit,
+                stellar_aberration=enable_stellar_aberration,
+            )
+        else:
+            ra_c, dec_c, _, _, _, _ = get_los(
+                observer,
+                track,
+                ts_mid,
+                deflection=enable_deflection,
+                aberration=enable_light_transit,
+                stellar_aberration=enable_stellar_aberration,
+            )
+
+        if fov_filter_radius is not None:
+            fov_half_diag_pad = fov_filter_radius
+        else:
+            y_ifov_os = y_fov / h_fpa_os
+            x_ifov_os = x_fov / w_fpa_os
+            y_fov_pad = y_fov + h_pad_os * y_ifov_os
+            x_fov_pad = x_fov + w_pad_os * x_ifov_os
+            fov_half_diag_pad = 0.5 * math.sqrt(y_fov_pad ** 2 + x_fov_pad ** 2)
+
+        # Convert angle threshold to cosine for efficient comparison
+        fov_half_diag_pad_cos = math.cos(math.radians(fov_half_diag_pad))
+    else:
+        fov_half_diag_pad = None
+        fov_half_diag_pad_cos = None
+        ra_c = dec_c = None
+
     for i, o in enumerate(obs):
 
         # TODO support in frame events
@@ -1104,12 +1184,41 @@ def _gen_objects(ssp, render_mode,
                     ts_epoch = time.utc_from_list_or_scalar(o['epoch'], default_t=tt)
                     obs_cache[i] = [create_twobody(np.array(o['position']) * u.km, np.array(o['velocity']) * u.km / u.s, ts_epoch)]
 
+            if az_arr is None:
+                az_arr, el_arr = _calculate_az_el(tc_start, tc_end, [ts_start, ts_mid, ts_end], track_az, track_el)
+                if enable_fov_filter:
+                    if track_mode == 'fixed':
+                        ra_c, dec_c, _, _, _, _ = get_los_azel(
+                            observer,
+                            az_arr[1],
+                            el_arr[1],
+                            ts_mid,
+                            deflection=enable_deflection,
+                            aberration=enable_light_transit,
+                            stellar_aberration=enable_stellar_aberration,
+                        )
+                    else:
+                        ra_c, dec_c, _, _, _, _ = get_los(
+                            observer,
+                            track,
+                            ts_mid,
+                            deflection=enable_deflection,
+                            aberration=enable_light_transit,
+                            stellar_aberration=enable_stellar_aberration,
+                        )
+
             o_offset = [0.0, 0.0]
             if 'offset' in o:
                 o_offset = [o['offset'][0] * h_fpa_os, o['offset'][1] * w_fpa_os]
 
             target = obs_cache[i][0]
-            az, el = _calculate_az_el(tc_start, tc_end, [ts_start, ts_mid, ts_end], track_az, track_el)
+            if enable_fov_filter:
+                # Determine the center line-of-sight of the sensor at mid-exposure
+                # Skip propagation if target is outside the padded field of view
+                # Use cosine comparison for efficiency (cos decreases as angle increases)
+                cos_ang = optimized_angle_from_los_cosine(observer, target, ra_c, dec_c, ts_mid)
+                if cos_ang < fov_half_diag_pad_cos:
+                    continue
 
             try:
                 [rr0, rr1, rr2], [cc0, cc1, cc2], _, _, _ = gen_track(
@@ -1129,11 +1238,11 @@ def _gen_objects(ssp, render_mode,
                     offset=o_offset,
                     flipud=ssp['fpa']['flip_up_down'],
                     fliplr=ssp['fpa']['flip_left_right'],
-                    az=az,
-                    el=el,
-                    deflection=ssp['sim']['enable_deflection'],
-                    aberration=ssp['sim']['enable_light_transit'],
-                    stellar_aberration=ssp['sim']['enable_stellar_aberration'],
+                    az=az_arr,
+                    el=el_arr,
+                    deflection=enable_deflection,
+                    aberration=enable_light_transit,
+                    stellar_aberration=enable_stellar_aberration,
                 )
             except Exception:
                 logger.exception("Error propagating target. {}".format(o))
